﻿# -*- coding: utf-8 -*-

import sys
import select
import re
import binascii
import struct

def default_callback(str_title, str_buffer):
  if len(str_buffer) > 0:
    print str_title
    print str_buffer
    print "==============================================="

def str2rawbuf(input_str):
  raw_struct = ""
  pkg_lines = input_str.split("\n");
  for str_line in pkg_lines:
    line = str_line.strip()
    if (line == ''): continue

    buf_match = re.match(r'[0x]*[0-9,A-F,a-f]{4,8}:\s*[0-9,A-F,a-f]{2,4}\W*', line)
    if buf_match is None: continue

    split_list = re.split(r'\s*:\s', line)
    if len(split_list) <= 1: continue

    hex_code = re.findall(r'[0-9,A-F,a-f]{2,2}', split_list[1])

    arr_cols = []
    for hex in hex_code[:16]:
      if len(hex) == 2:
        arr_cols.append(hex)

    hex_line = [ord(binascii.unhexlify(v)) for v in arr_cols if v != '']
    raw_struct += struct.pack('B'*len(hex_line), *hex_line)

  return raw_struct

class StdinTcpdump:
    """ read tcpdump input from stdin """
    last_line = ""
    result_str = ""
    callback = None

    def __init__(self, callback_func = default_callback):
      self.callback = callback_func
      if (self.callback == None):
        raise Exception("callback unknow")
    
    def process_string_line(self, input_str):
      """ 处理输入行，返回正数表示读取到空行或非数据行，返回0表示数据行 """
      line = input_str.strip()
      if len(line) == 0:
        return 1

      # 匹配 (0x)XXXX: XX XX XX XX ... 的有效行，非有效行则认为读取终止
      buf_match = re.match(r'^[0x]*[0-9,A-F,a-f]{4,8}:\s*[0-9,A-F,a-f]{2,4}\W*', line)
      if buf_match is None:
        self.callback(self.last_line, self.result_str)
        self.result_str = ""
        self.last_line = line
        return 2

      self.result_str += input_str
      return 0

    def run(self):
      """ read buffer form stdin """
      import time

      while True:
        raw_line = sys.stdin.readline()
        if not raw_line: break
        
        if (self.process_string_line(raw_line) != 0):
          sys.stdout.flush()
          time.sleep(0.001)

      # flush last buffer
      self.process_string_line("END")

# [DEBUG] tcpdump -l <args> | python -u <args>
#stdin_process = StdinTcpdump(default_callback)
#stdin_process.run()
